import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .camera_config import *
from .camera import *
from .processing_config import *


class IntegrationTime:
    """
    The integration time as fixed presets
    """

    SHORT = 0
    "250 us"

    MEDIUM = 1
    "500 us"

    LONG = 2
    "1000us"



class ImagingEnvironment:
    """
    The different imaging environments
    """

    INSIDE = 0
    "Inside buildings"

    SUNLIGHT = 1
    "When bright sunlight is present"



class Strategy:
    """
    The optimization of the camera configuration
    """

    BALANCED = 0
    "The balanced configuration"

    SPEED = 1
    "Optimize for moving objects"

    ACCURACY = 2
    "Optimize for high accuracy"



class ConfigMode:
    """
    Optimized configurations for different use cases
    """

    LONG_RANGE_DYNAMIC_SCENE = 1
    "Suitable for most applications"

    MEDIUM_RANGE_DYNAMIC_SCENE = 2
    "For fast moving applications"

    MEDIUM_RANGE_BALANCED = 3
    "Balanced between range, speed and accuracy"

    SHORT_RANGE_HIGH_SPEED = 4
    "Maximum speed mode for short range and fast moving applications"

    LONG_RANGE_HIGH_RESOLUTION = 5
    "Full resolution long range"

    MEDIUM_RANGE_STATIC_SCENE = 6
    "Full resolution for slow changing scene"

    FAR_OUT_STATIC_SCENE = 7
    "Full resolution for slow changing scene"



class UserConfig():
    """
    DEPRECATED, use config_mode. Simple interface for camera configuration
    generation
    """
    def __del__(self):
        if self._ptr:
            lib.tof_user_config_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new user_config class
        ----------

        Create default user config
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            self._ptr = lib.tof_user_config_new_default(get_error())


    def toCameraConfig(self, camera: Camera) -> CameraConfig:
        """
        Convert to a camera configuration

        Parameters
        ----------
        camera : Camera
            The camera

        Returns
        ----------
        CameraConfig
            The camera configuration
        """
        return CameraConfig(ptr = lib.tof_user_config_to_camera_config(self._ptr, camera._ptr, get_error()))


    def getEnvironment(self) -> ImagingEnvironment:
        """
        Get the camera operating environment

        Returns
        ----------
        ImagingEnvironment
            Imaging environment
        """
        return lib.tof_user_config_get_environment(self._ptr, get_error())


    def setEnvironment(self, imaging_environment: ImagingEnvironment) -> None:
        """
        Set the camera operating environment

        Parameters
        ----------
        imaging_environment : ImagingEnvironment
            Imaging environment
        """
        return lib.tof_user_config_set_environment(self._ptr, imaging_environment, get_error())


    def getMaxDistance(self) -> float:
        """
        Get the maximum distance photons reflect from

        Returns
        ----------
        float
            Maximum distance in meters
        """
        return lib.tof_user_config_get_max_distance(self._ptr, get_error())


    def setMaxDistance(self, max_distance: float) -> None:
        """
        Set the maximum distance photons reflect from

        Parameters
        ----------
        max_distance : float
            Maximum distance in meters
        """
        return lib.tof_user_config_set_max_distance(self._ptr, max_distance, get_error())


    def getFps(self) -> float:
        """
        Get the depth frame rate

        Returns
        ----------
        float
            The depth fps
        """
        return lib.tof_user_config_get_fps(self._ptr, get_error())


    def setFps(self, fps: float) -> None:
        """
        Set the depth frame rate

        Parameters
        ----------
        fps : float
            The depth fps
        """
        return lib.tof_user_config_set_fps(self._ptr, fps, get_error())


    def getLensFlare(self) -> bool:
        """
        Get if lens flare is expected in the scene

        Returns
        ----------
        bool
            if lens flare is present
        """
        return lib.tof_user_config_get_lens_flare(self._ptr, get_error())


    def setLensFlare(self, lens_flare: bool) -> None:
        """
        Set if lens flare is expected in the scene

        Parameters
        ----------
        lens_flare : bool
            if lens flare is present
        """
        return lib.tof_user_config_set_lens_flare(self._ptr, lens_flare, get_error())


    def getTranslucent(self) -> bool:
        """
        Get if there are translucent objects in the scene

        Returns
        ----------
        bool
            If translucent objects
        """
        return lib.tof_user_config_get_translucent(self._ptr, get_error())


    def setTranslucent(self, translucent: bool) -> None:
        """
        Set if there are translucent objects in the scene

        Parameters
        ----------
        translucent : bool
            If translucent objects
        """
        return lib.tof_user_config_set_translucent(self._ptr, translucent, get_error())


    def getHdr(self) -> bool:
        """
        Get high dynamic range enabled. Enable this when pixel saturation occurs.
        This will only happen for retroreflectors or reflective objects close
        (<20cm) to the camera

        Returns
        ----------
        bool
            Enable high dynamic range
        """
        return lib.tof_user_config_get_hdr(self._ptr, get_error())


    def setHdr(self, hdr: bool) -> None:
        """
        Set high dynamic range enabled. Enable this when pixel saturation occurs.
        This will only happen for retroreflectors or reflective objects close
        (<20cm) to the camera

        Parameters
        ----------
        hdr : bool
            Enable high dynamic range
        """
        return lib.tof_user_config_set_hdr(self._ptr, hdr, get_error())


    def getIntegrationTime(self) -> IntegrationTime:
        """
        Get the integration time

        Returns
        ----------
        IntegrationTime
            The integration time
        """
        return lib.tof_user_config_get_integration_time(self._ptr, get_error())


    def setIntegrationTime(self, int_time: IntegrationTime) -> None:
        """
        Set the integration time

        Parameters
        ----------
        int_time : IntegrationTime
            The integration time
        """
        return lib.tof_user_config_set_integration_time(self._ptr, int_time, get_error())


    def getStrategy(self) -> Strategy:
        """
        Get what the configuration is being optimized for

        Returns
        ----------
        Strategy
            The user configuration strategy
        """
        return lib.tof_user_config_get_strategy(self._ptr, get_error())


    def setStrategy(self, strategy: Strategy) -> None:
        """
        Set what the configuration is being optimized for

        Parameters
        ----------
        strategy : Strategy
            The user configuration strategy
        """
        return lib.tof_user_config_set_strategy(self._ptr, strategy, get_error())


    def getChannel(self) -> int:
        """
        Get The camera channel, increment to support multi-camera usage

        Returns
        ----------
        int
            The channel number, set to 0, otherwise increment with number of
            cameras.
        """
        return lib.tof_user_config_get_channel(self._ptr, get_error())


    def setChannel(self, channel: int) -> None:
        """
        Set The camera channel, increment to support multi-camera usage

        Parameters
        ----------
        channel : int
            The channel number, set to 0, otherwise increment with number of
            cameras.
        """
        return lib.tof_user_config_set_channel(self._ptr, channel, get_error())


def configModeCameraConfig(camera: Camera, config_mode: ConfigMode) -> CameraConfig:
    """
    Get the camera config associated with the config mode

    Parameters
    ----------
    camera : Camera
        Camera
    config_mode : ConfigMode
        Config mode

    Returns
    ----------
    CameraConfig
        Camera Config
    """
    return CameraConfig(ptr = lib.tof_config_mode_camera_config(camera._ptr, config_mode, get_error()))


def configModeProcessingConfig(camera: Camera, config_mode: ConfigMode) -> ProcessingConfig:
    """
    Get the processing config associated with the config mode

    Parameters
    ----------
    camera : Camera
        Camera
    config_mode : ConfigMode
        Config mode

    Returns
    ----------
    ProcessingConfig
        Processing Config
    """
    return ProcessingConfig(ptr = lib.tof_config_mode_processing_config(camera._ptr, config_mode, get_error()))


def hdrPossible(config: CameraConfig) -> bool:
    """
    Check whether hdr is possible with current camera config

    Parameters
    ----------
    config : CameraConfig
        The camera config to check

    Returns
    ----------
    bool
        Whether hdr is possible
    """
    return lib.tof_hdr_possible(config._ptr, get_error())


def phaseUnwrapPossible(config: CameraConfig) -> bool:
    """
    Check whether phase unwrap is possible with current camera config

    Parameters
    ----------
    config : CameraConfig
        The camera config to check

    Returns
    ----------
    bool
        Whether phase unwrap is possible
    """
    return lib.tof_phase_unwrap_possible(config._ptr, get_error())


def applyDefaultProcessingConfig(camera: Camera) -> None:
    """
    Generate processing config based on camera configuration and calibration.

    Parameters
    ----------
    camera : Camera
        A camera
    """
    return lib.tof_apply_default_processing_config(camera._ptr, get_error())


